import { Component, OnInit, ViewChild } from "@angular/core";
import { ActivatedRoute } from "@angular/router";
import { FormBuilder, Validators, FormGroup } from "@angular/forms";
import { NzMessageService } from "ng-zorro-antd/message";

import { MainService } from "../../services/main.service";
import { OverlayScrollbarsComponent } from "overlayscrollbars-ngx";
import { ToolService } from "../../services/tool.service";
import { GenerateFloorComponent } from "src/app/share/generate-floor/generate-floor.component";

@Component({
  selector: "app-building-floor",
  templateUrl: "./building-floor.component.html",
  styleUrls: ["./building-floor.component.less"],
})
export class BuildingFloorComponent implements OnInit {
  tableHeight;

  constructor(
    private fb: FormBuilder,
    private route: ActivatedRoute,
    private mainService: MainService,
    private msg: NzMessageService,
    private tool: ToolService
  ) {}
  @ViewChild("osComponentRef1", {
    read: OverlayScrollbarsComponent,
    static: false,
  })
  osComponentRef1: OverlayScrollbarsComponent;
  @ViewChild(GenerateFloorComponent, { static: false })
  generate1: GenerateFloorComponent;
  ngOnInit() {
    this.coopBtns = this.tool.initCoopBtns(this.route);
    console.log(this.coopBtns);
    this.initForm();
    this.initFormFloor();
    this.getHospitalList();
    this.getBuildingList();
    this.tableHeight = document.body.clientHeight - 267;
  }
  isAllDisplayDataChecked = false;
  isIndeterminate = false;
  listOfDisplayData: any[] = [];
  allFloorList: any[] = []; //当前选中的楼栋包含的所有楼层
  mapOfCheckedId: { [key: string]: boolean } = {};
  checkedBuilding: any = {}; //选中楼栋
  buildingList: Array<any> = []; //楼栋信息
  hospitalList: Array<any> = []; //院区列表
  hosId: any; //当前选择的院区id

  promptContent: string; //操作提示框提示信息
  ifSuccess: boolean; //操作成功/失败
  promptInfo: string; //操作结果提示信息
  promptModalShow: boolean; //是否展示提示框

  btnLoading: boolean = false; //确认按钮loading状态
  saveLoading: boolean = false; //批量打印按钮loading状态

  // 初始化增删改按钮
  coopBtns: any = {};

  // 分组列表
  loading1 = false;
  getBuildingList() {
    let postData = {
      idx: 0,
      sum: 9999,
      building: { hosId: this.hosId },
    };
    this.loading1 = true;
    this.mainService
      .getFetchDataList("simple/data", "building", postData)
      .subscribe((result) => {
        this.loading1 = false;
        if (result.status == 200) {
          this.buildingList = result.list;
          if (Object.keys(this.checkedBuilding).length) {
            result.list.forEach((item) => {
              if (item.id == this.checkedBuilding.id) {
                this.checkGroup(item);
              }
            });
          } else {
            this.checkGroup(result.list[0]);
          }
        }
      });
  }

  // 获取选中楼栋下楼层列表
  loading2 = false;
  allUsers = [];
  getAllFloor() {
    // 初始化的时候搜索一次，然后前端过滤
    let postData = {
      idx: 0,
      sum: 9999,
      floor: { hosId: this.hosId, buildId: this.checkedBuilding.id },
    };
    this.loading2 = true;
    this.mainService
      .getFetchDataList("simple/data", "floor", postData)
      .subscribe((result) => {
        this.loading2 = false;
        if (result.status == 200) {
          this.allFloorList = result.list;
          this.allUsers = JSON.parse(JSON.stringify(result.list));
          this.refreshStatus(true);
        }
      });
  }

  // 院区列表
  getHospitalList() {
    this.hosId = this.tool.getCurrentHospital().id;
    this.hospitalList = [this.tool.getCurrentHospital()];
  }
  // 选中分组
  positionY = 0; //记录其他建单Y轴滚动距离
  checkGroup(data) {
    this.positionY = this.osComponentRef1.osInstance().scroll().position.y; //内容滚动的距离
    this.checkedBuilding = data;
    this.mapOfCheckedId = {};
    this.getAllFloor();
  }
  selectedUser(data) {
    this.mapOfCheckedId[data.id] = !this.mapOfCheckedId[data.id];
    this.refreshStatus();
  }
  // 选中列表中当前分组用户
  usersArr = [];
  refreshStatus(first?): void {
    let arr = [];
    for (var m in this.mapOfCheckedId) {
      if (this.mapOfCheckedId[m]) {
        arr.push({ id: m });
      }
    }
    this.usersArr = arr;
  }

  // 新增/编辑楼栋模态框
  coopModal: boolean = false; //模态框是否展示
  add: boolean = true; //true：新增；false：编辑
  showCoopModal(type) {
    this.coopModal = true;
    this.add = type == "add";
    if (type == "edit") {
      this.validateForm.controls.buildingName.setValue(
        this.checkedBuilding["buildingName"]
      );
    } else {
      this.validateForm.controls.buildingName.setValue(null);
    }
  }
  // 新增/编辑楼层模态框
  floorModal: boolean = false; //模态框是否展示
  addFloor: boolean = true; //true：新增；false：编辑
  floorDataEdit; //正在编辑的楼层
  showFloorModal(e, type, data?) {
    this.floorModal = true;
    this.addFloor = type == "add";
    if (type == "edit") {
      this.floorDataEdit = data;
      this.validateFloorForm.controls.buildId.setValue(data.buildId);
      this.validateFloorForm.controls.floorName.setValue(data.floorName);
    } else {
      this.validateFloorForm.controls.buildId.setValue(this.checkedBuilding.id);
      this.validateFloorForm.controls.floorName.setValue(null);
    }
    e.stopPropagation();
  }
  // 隐藏楼栋模态框
  hideCoopModal() {
    this.coopModal = false;
  }
  // 隐藏楼层模态框
  hideFloorModal() {
    this.floorModal = false;
  }

  // 初始化新增form表单
  initForm() {
    this.validateForm = this.fb.group({
      buildingName: [null, [Validators.required]],
    });
  }
  // 初始化新增form表单floor
  validateFloorForm: FormGroup;
  initFormFloor() {
    this.validateFloorForm = this.fb.group({
      buildId: [null, [Validators.required]],
      floorName: [null, [Validators.required]],
    });
  }

  // 新增/编辑楼栋提交
  validateForm: FormGroup;
  submitForm(): void {
    for (const i in this.validateForm.controls) {
      this.validateForm.controls[i].markAsDirty();
      this.validateForm.controls[i].updateValueAndValidity();
    }
    if (this.validateForm.invalid) return;
    this.btnLoading = true;
    let postData;
    if (this.add) {
      let arr = this.buildingList.filter(
        (item) => item.buildingName == this.validateForm.value.buildingName
      );
      //有重复名称
      if (arr.length > 0) {
        this.btnLoading = false;
        this.showPromptModal(
          "新增",
          false,
          `存在重复的楼栋名称【${this.validateForm.value.buildingName}】请修改后再保存！`
        );
        return;
      }
      postData = {
        buildingName: this.validateForm.value.buildingName,
        deleted: false,
        hosId: this.hosId,
      };
    } else {
      postData = {
        buildingName: this.validateForm.value.buildingName,
        deleted: false,
        hosId: this.hosId,
        id: this.checkedBuilding.id,
      };
    }
    this.mainService
      .simplePost("addData", "building", postData)
      .subscribe((result) => {
        this.hideCoopModal();
        this.btnLoading = false;
        if (result["status"] == 200) {
          this.showPromptModal(this.add ? "新增" : "编辑", true, "");
        } else {
          this.showPromptModal(
            this.add ? "新增" : "编辑",
            false,
            result["msg"]
          );
        }
      });
  }
  // 新增/编辑楼层提交
  submitFormFloor(): void {
    for (const i in this.validateFloorForm.controls) {
      this.validateFloorForm.controls[i].markAsDirty();
      this.validateFloorForm.controls[i].updateValueAndValidity();
    }
    if (this.validateFloorForm.invalid) return;
    this.btnLoading = true;
    let postData;
    if (this.addFloor) {
      let arr = this.allFloorList.filter(
        (item) => item.floorName == this.validateFloorForm.value.floorName
      );
      //有重复名称
      if (arr.length > 0) {
        this.btnLoading = false;
        this.showPromptModal(
          "新增",
          false,
          `同一个楼栋存在重复的楼层名称【${this.validateFloorForm.value.floorName}】请修改后再保存！`
        );
        return;
      }
      postData = {
        buildId: this.validateFloorForm.value.buildId,
        floorName: this.validateFloorForm.value.floorName,
        deleted: false,
        hosId: this.hosId,
      };
    } else {
      postData = {
        buildId: this.validateFloorForm.value.buildId,
        floorName: this.validateFloorForm.value.floorName,
        deleted: false,
        hosId: this.hosId,
        id: this.floorDataEdit.id,
      };
    }
    this.addFloorHandler(postData, false);
  }
  //新增楼层
  addFloorHandler(postData, flag) {
    this.mainService
      .simplePost(flag ? "addListData" : "addData", "floor", postData)
      .subscribe((result) => {
        this.hideFloorModal();
        this.generate1.delModal = false;
        this.hideGenerateModal();
        this.btnLoading = false;
        if (result["status"] == 200) {
          this.showPromptModal(this.addFloor ? "新增" : "编辑", true, "");
        } else {
          this.showPromptModal(
            this.addFloor ? "新增" : "编辑",
            false,
            result["msg"]
          );
        }
      });
  }

  //删除楼栋
  delModal: boolean = false;
  showDelModal() {
    this.delModal = true;
  }
  hideDelModal() {
    this.delModal = false;
  }
  // 确认删除
  confirmDel() {
    this.btnLoading = true;
    let postData = [this.checkedBuilding["id"]];
    this.mainService.delBuildingList(postData).subscribe((result) => {
      this.hideDelModal();
      this.btnLoading = false;
      if (result["status"] == 200 && !result["data"][0].msg) {
        this.showPromptModal("删除", true, "");
      } else {
        this.showPromptModal("删除", false, result["data"][0].msg);
      }
    });
  }
  /**
   * 删除楼层
   * @param e 事件对象
   * @param data 有值就是单个删除，无值就是批量删除
   */
  delFloorModal: boolean = false;
  isDelSingle: boolean = false; //是否单个删除
  showDelFloorModal(e, data?) {
    if (data) {
      this.isDelSingle = true;
      this.floorDataEdit = data;
    } else {
      this.isDelSingle = false;
    }
    this.delFloorModal = true;
    e.stopPropagation();
  }
  hideDelFloorModal() {
    this.delFloorModal = false;
  }
  // 确认删除
  confirmFloorDel() {
    this.btnLoading = true;
    let ids = Object.keys(this.mapOfCheckedId); //批量删除选中的楼层
    let postData = this.isDelSingle ? [this.floorDataEdit.id] : ids;
    this.mainService.delFloorList(postData).subscribe((result) => {
      this.hideDelFloorModal();
      this.btnLoading = false;
      if (result["status"] == 200 && !result["data"][0].msg) {
        this.showPromptModal("删除", true, "");
      } else {
        this.showPromptModal("删除", false, result["data"][0].msg);
      }
    });
  }

  // 展示信息提示框（con:提示信息，success:操作是否成功，promptInfo:操作结果提示信息）（con:提示信息，success:操作是否成功，promptInfo:操作结果提示信息）
  showPromptModal(con, success, promptInfo) {
    this.promptModalShow = false;
    this.promptContent = con;
    this.ifSuccess = success;
    this.promptInfo = promptInfo;
    this.osComponentRef1.osInstance().scroll({ x: 0, y: this.positionY });
    setTimeout(() => {
      this.promptModalShow = true;
      this.getBuildingList();
    }, 100);
  }
  // 生成楼层
  generateModal: boolean = false;
  generate() {
    this.generateModal = true;
  }
  // 隐藏楼层模态框
  hideGenerateModal() {
    this.generateModal = false;
  }
  // 确认生成
  confirmGenerate(e) {
    console.log(e);
    let arr = []; //生成的楼层
    for (let i = e[1]; i <= e[2]; i++) {
      arr.push({
        buildId: e[0].id,
        floorName: i + "",
        deleted: false,
        hosId: this.hosId,
      });
    }
    // 去重
    this.allFloorList.forEach((item1) => {
      arr = arr.filter(
        (item2) =>
          !(
            item1.buildId == item2.buildId && item1.floorName == item2.floorName
          )
      );
    });
    this.btnLoading = true;
    this.addFloorHandler(arr, true);
  }
}
